from shippo.resource import (
    Address,
    Batch,
    CarrierAccount,
    CustomsDeclaration,
    CustomsItem,
    Invoice,
    InvoiceItem,
    LineItem,
    Manifest,
    Order,
    Parcel,
    Pickup,
    Rate,
    Refund,
    Shipment,
    Track,
    Transaction,
    Webhook,
)
from shippo.resource import (V2Addresses)


# -*- coding: utf-8 -*-
import unittest

from mock import patch
from datetime import datetime, timedelta

import shippo
from shippo.test.helper import (
    create_mock_international_transaction,
    ShippoTestCase,
    DUMMY_PICKUP
)

from shippo.test.helper import shippo_vcr


class PickupTests(ShippoTestCase):
    request_client = shippo.http_client.RequestsClient

    def setUp(self):
        super(PickupTests, self).setUp()

        def get_http_client(*args, **kwargs):
            return self.request_client(*args, **kwargs)

        self.client_patcher = patch(
            'shippo.http_client.new_default_http_client')

        client_mock = self.client_patcher.start()
        client_mock.side_effect = get_http_client

    def tearDown(self):
        super(PickupTests, self).tearDown()

        self.client_patcher.stop()

    @shippo_vcr.use_cassette(cassette_library_dir='shippo/test/fixtures/pickup')
    def test_create(self):
        transaction, carrier_account = create_mock_international_transaction()
        PICKUP = DUMMY_PICKUP
        PICKUP['carrier_account'] = carrier_account
        PICKUP['transactions'] = [transaction.object_id]
        pickupTimeStart = datetime.now() + timedelta(hours=1)
        pickupTimeEnd = pickupTimeStart + timedelta(days=1)
        PICKUP['pickupTimeStart'] = pickupTimeStart.isoformat() + "Z"
        PICKUP['pickupTimeEnd'] = pickupTimeEnd.isoformat() + "Z"
        try:
            pickup = shippo.Pickup.create(**PICKUP)
        except shippo.error.InvalidRequestError:
            self.assertTrue(True)
        else:
            self.assertEqual(pickup.status, 'SUCCESS')

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import unittest

from mock import patch
from datetime import datetime

import shippo
from shippo.test.helper import (
    ShippoTestCase,
    DUMMY_ORDER
)

from shippo.test.helper import shippo_vcr


class OrderTests(ShippoTestCase):
    request_client = shippo.http_client.RequestsClient

    def setUp(self):
        super(OrderTests, self).setUp()

        def get_http_client(*args, **kwargs):
            return self.request_client(*args, **kwargs)

        self.client_patcher = patch(
            'shippo.http_client.new_default_http_client')

        client_mock = self.client_patcher.start()
        client_mock.side_effect = get_http_client

    def tearDown(self):
        super(OrderTests, self).tearDown()

        self.client_patcher.stop()

    @shippo_vcr.use_cassette(cassette_library_dir='shippo/test/fixtures/order')
    def test_invalid_create(self):
        self.assertRaises(shippo.error.InvalidRequestError,
                          shippo.Order.create)

    @shippo_vcr.use_cassette(cassette_library_dir='shippo/test/fixtures/order')
    def test_create(self):
        ORDER = DUMMY_ORDER
        ORDER['placed_at'] = datetime.now().isoformat() + "Z"
        order = shippo.Order.create(**ORDER)
        self.assertEqual(order.order_status, 'PAID')

    @shippo_vcr.use_cassette(cassette_library_dir='shippo/test/fixtures/order')
    def test_retrieve(self):
        ORDER = DUMMY_ORDER
        ORDER['placed_at'] = datetime.now().isoformat() + "Z"
        order = shippo.Order.create(**ORDER)
        retrieve = shippo.Order.retrieve(order.object_id)
        self.assertEqual(order.object_id, retrieve.object_id)

    @shippo_vcr.use_cassette(cassette_library_dir='shippo/test/fixtures/order')
    def test_invalid_retrieve(self):
        self.assertRaises(shippo.error.APIError,
                          shippo.Order.retrieve, 'EXAMPLE_OF_INVALID_ID')

    @shippo_vcr.use_cassette(cassette_library_dir='shippo/test/fixtures/order')
    def test_list_all(self):
        order_list = shippo.Order.all()
        self.assertTrue('results' in order_list)

    @shippo_vcr.use_cassette(cassette_library_dir='shippo/test/fixtures/order')
    def test_list_page_size(self):
        pagesize = 1
        order_list = shippo.Order.all(size=pagesize)
        self.assertEqual(len(order_list.results), pagesize)

if __name__ == '__main__':
    unittest.main()



import os
from typing import Optional

from shippo.version import VERSION


class Configuration:
    def __init__(self):
        self.sdk_version: str = VERSION
        self.api_base: str = os.environ.get('SHIPPO_API_BASE', 'https://api.goshippo.com/')
        self.api_key: str = os.environ.get('SHIPPO_API_KEY')
        self.api_version: str = os.environ.get('SHIPPO_API_VERSION', '2018-02-08')

        self.app_name: str = os.environ.get('APP_NAME', '')
        self.app_version: str = os.environ.get('APP_VERSION', '')

        self.sdk_name = 'ShippoPythonSDK'
        self.language = 'Python'

        self.verify_ssl_certs = True
        self.timeout_in_seconds: Optional[float] = None
        self.rates_req_timeout = float(os.environ.get('RATES_REQ_TIMEOUT', 20.0))

        # SETTINGS BELOW APPLY ONLY TO TESTS

        # Controls how much information is logged to stdout
        self.vcr_logging_level = os.environ.get('VCR_LOGGING_LEVEL', 'ERROR')

        # Switch to 'all' for re-recording all cassettes (to be used in CI so that all tests are run without mocks)
        self.vcr_record_mode = os.environ.get('VCR_RECORD_MODE', 'once')


config = Configuration()



import datetime
import os
import re
import shippo
from shippo.config import config
import vcr

from mock import patch, Mock
from unittest import TestCase

NOW = datetime.datetime.now()

DUMMY_ADDRESS = {
    "name": "Laura Behrens Wu",
    "company": "Shippo",
    "street1": "Clayton St.",
    "street_no": "215",
    "street2": "",
    "city": "San Francisco",
    "state": "CA",
    "zip": "94117",
    "country": "US",
    "phone": "+1 555 341 9393",
    "metadata": "Customer ID 123456"
}
INVALID_ADDRESS = {
    "name": "Laura Behrens Wu",
    "company": "Shippo",
    "street2": "",
    "city": "San Francisco",
    "state": "CA",
    "country": "US",
    "phone": "+1 555 341 9393",
    "metadata": "Customer ID 123456"
}
NOT_POSSIBLE_ADDRESS = {
    "name": "Laura Behrens Wu",
    "company": "Shippo",
    "street1": "ClaytonKLJLKJL St.",
    "street_no": "0798987987987",
    "street2": "",
    "city": "San Francisco",
    "state": "CA",
    "zip": "74338",
    "country": "US",
    "phone": "+1 555 341 9393",
    "metadata": "Customer ID 123456"
}
DUMMY_PARCEL = {
    "length": "5",
    "width": "5",
    "height": "5",
    "distance_unit": "cm",
    "weight": "2",
    "mass_unit": "lb",
    "template": "",
    "metadata": "Customer ID 123456"
}
INVALID_PARCEL = {
    "length": "5",
    "width": "5",
    "distance_unit": "cm",
    "weight": "2",
    "template": "",
    "metadata": "Customer ID 123456"
}
DUMMY_MANIFEST = {
    "provider": "USPS",
    "shipment_date": "2017-03-31T17:37:59.817Z",
    "address_from": "28828839a2b04e208ac2aa4945fbca9a"
}
INVALID_MANIFEST = {
    "provider": "RANDOM_INVALID_PROVIDER",
    "shipment_date": "2014-05-16T23:59:59Z",
    "address_from": "EXAMPLE_OF_INVALID_ADDRESS"
}
DUMMY_CUSTOMS_ITEM = {
    "description": "T-Shirt",
    "quantity": 2,
    "net_weight": "400",
    "mass_unit": "g",
    "value_amount": "20",
    "value_currency": "USD",
    "tariff_number": "",
    "origin_country": "US",
    "metadata": "Order ID #123123"
}
INVALID_CUSTOMS_ITEM = {
    "value_currency": "USD",
    "tariff_number": "",
    "origin_country": "US",
    "metadata": "Order ID #123123"
}
DUMMY_CUSTOMS_DECLARATION = {
    "exporter_reference": "",
    "importer_reference": "",
    "contents_type": "MERCHANDISE",
    "contents_explanation": "T-Shirt purchase",
    "invoice": "#123123",
    "license": "",
    "certificate": "",
    "notes": "",
    "eel_pfc": "NOEEI_30_37_a",
    "aes_itn": "",
    "non_delivery_option": "ABANDON",
    "certify": True,
    "certify_signer": "Laura Behrens Wu",
    "disclaimer": "",
    "incoterm": "",
    "items": [
        "0c1a723687164307bb2175972fbcd9ef"
    ],
    "metadata": "Order ID #123123"
}
INVALID_CUSTOMS_DECLARATION = {
    "exporter_reference": "",
    "importer_reference": "",
    "contents_type": "MERCHANDISE",
    "contents_explanation": "T-Shirt purchase",
    "invoice": "#123123",
    "license": "",
    "certificate": "",
    "notes": "",
    "eel_pfc": "NOEEI_30_37_a",
    "aes_itn": "",
    "non_delivery_option": "ABANDON",
    "cerfy": True,
    "certify_signer": "Laura Behrens Wu",
    "disclaimer": "",
    "incoterm": "",
    "metadata": "Order ID #123123"
}
TO_ADDRESS = {
    "name": "John Smith",
    "company": "Initech",
    "street1": "965 Mission Street",
    "street_no": "",
    "street2": "Ste 480",
    "city": "San Francisco",
    "state": "CA",
    "zip": "94103",
    "country": "US",
    "phone": "+1 630 333 7333",
    "metadata": "Customer ID 123456"
}
FROM_ADDRESS = {
    "name": "Laura Behrens Wu",
    "company": "Shippo",
    "street1": "Clayton St.",
    "street_no": "215",
    "street2": "",
    "city": "San Francisco",
    "state": "CA",
    "zip": "94117",
    "country": "US",
    "phone": "+1 555 341 9393",
    "metadata": "Customer ID 123456"
}
DUMMY_SHIPMENT = {
    "address_from": "4f406a13253945a8bc8deb0f8266b245",
    "address_to": "4c7185d353764d0985a6a7825aed8ffb",
    "parcels": ["ec952343dd4843c39b42aca620471fd5"],
    "submission_type": "PICKUP",
    "insurance_amount": "200",
    "insurance_currency": "USD",
    "extra": {
        "signature_confirmation": True,
        "reference_1": "",
        "reference_2": "",
        "insurance": {
            "amount": "200",
            "currency": "USD"
        }
    },
    "metadata": "Customer ID 123456"
}
DUMMY_INTERNATIONAL_SHIPMENT = {
    "address_from": "4f406a13253945a8bc8deb0f8266b245",
    "address_to": "4c7185d353764d0985a6a7825aed8ffb",
    "parcels": ["ec952343dd4843c39b42aca620471fd5"],
    "submission_type": "PICKUP",
    "insurance_amount": "200",
    "insurance_currency": "USD",
    "extra": {
        "signature_confirmation": True,
        "reference_1": "",
        "reference_2": "",
        "insurance": {
            "amount": "200",
            "currency": "USD"
        }
    },
    "metadata": "Customer ID 123456"
}
INVALID_SHIPMENT = {
    "address_from": "4f406a13253945a8bc8deb0f8266b245",
    "submission_type": "PICKUP",
    "shipment_date": "2017-03-31T17:37:59.817Z",
    "extra": {
        "signature_confirmation": True,
        "reference_1": "",
        "reference_2": "",
        "insurance": {
            "amount": "200",
            "currency": "USD"
        }
    },
    "customs_declaration": "b741b99f95e841639b54272834bc478c",
    "metadata": "Customer ID 123456"
}
DUMMY_TRANSACTION = {
    "rate": "67891d0ebaca4973ae2569d759da6139",
    "metadata": "Customer ID 123456"
}
INVALID_TRANSACTION = {
    "metadata": "Customer ID 123456"
}
DUMMY_BATCH = {
    "default_carrier_account": "e68e95b95e33431a87bdecdd2b891c2b",
    "default_servicelevel_token": "usps_priority",
    "label_filetype": "PDF_4x6",
    "metadata": "BATCH #170",
    "batch_shipments": [
        {
            "shipment": {
                "address_from": {
                    "name": "Mr Hippo",
                    "street1": "965 Mission St",
                    "street2": "Ste 201",
                    "city": "San Francisco",
                    "state": "CA",
                    "zip": "94103",
                    "country": "US",
                    "phone": "4151234567",
                },
                "address_to": {
                    "name": "Mrs Hippo",
                    "company": "",
                    "street1": "Broadway 1",
                    "street2": "",
                    "city": "New York",
                    "state": "NY",
                    "zip": "10007",
                    "country": "US",
                    "phone": "4151234567",
                },
                "parcels": [{
                    "length": "5",
                    "width": "5",
                    "height": "5",
                    "distance_unit": "in",
                    "weight": "2",
                    "mass_unit": "oz"
                }]
            }
        },
        {
            "shipment": {
                "address_from": {
                    "name": "Mr Hippo",
                    "street1": "1092 Indian Summer Ct",
                    "city": "San Jose",
                    "state": "CA",
                    "zip": "95122",
                    "country": "US",
                    "phone": "4151234567",
                },
                "address_to": {
                    "name": "Mrs Hippo",
                    "company": "",
                    "street1": "Broadway 1",
                    "street2": "",
                    "city": "New York",
                    "state": "NY",
                    "zip": "10007",
                    "country": "US",
                    "phone": "4151234567",
                },
                "parcels": [{
                    "length": "5",
                    "width": "5",
                    "height": "5",
                    "distance_unit": "in",
                    "weight": "2",
                    "mass_unit": "oz"
                }]
            }
        }
    ]
}
INVALID_BATCH = {
    "default_carrier_account": "NOT_VALID",
    "default_servicelevel_token": "usps_priority",
    "label_filetype": "PDF_4x6",
    "metadata": "teehee",
    "batch_shipments": []
}
DUMMY_PICKUP = {
        "carrier_account": "abcdefghijklmnopqrstuvwxyz0123456789",
        "location": {
            "building_location_type" : "Knock on Door",
            "address" : {
                "name": "Laura Behrens Wu",
                "company": "Shippo",
                "street1": "Clayton St.",
                "street_no": "215",
                "street2": "",
                "city": "San Francisco",
                "state": "CA",
                "zip": "94117",
                "country": "US",
                "phone": "+1 555 341 9393",
                "metadata": "Customer ID 123456"
            },
        },
        "transactions": [ "abcdefghijklmnopqrstuvwxyz0123456789" ],
        "requested_start_time": "2022-01-01T00:00:00.00Z",
        "requested_end_time": "2022-01-02T00:00:00.000Z",
        "is_test": False
}
DUMMY_ORDER = {
            "to_address": {
                "city": "San Francisco",
                "company": "Shippo",
                "country": "US",
                "email": "shippotle@goshippo.com",
                "name": "Mr Hippo",
                "phone": "15553419393",
                "state": "CA",
                "street1": "215 Clayton St.",
                "zip": "94117"
            },
            "line_items": [
                {
                    "quantity": 1,
                    "sku": "HM-123",
                    "title": "Hippo Magazines",
                    "total_price": "12.10",
                    "currency": "USD",
                    "weight": "0.40",
                    "weight_unit": "lb"
                }
            ],
            "placed_at": "2022-01-01T00:00:00.000Z",
            "order_number": "#1068",
            "order_status": "PAID",
            "shipping_cost": "12.83",
            "shipping_cost_currency": "USD",
            "shipping_method": "USPS First Class Package",
            "subtotal_price": "12.10",
            "total_price": "24.93",
            "total_tax": "0.00",
            "currency": "USD",
            "weight": "0.40",
            "weight_unit": "lb"
}


def create_mock_shipment(asynchronous=False, api_key=None):
    to_address = shippo.Address.create(api_key=api_key, **TO_ADDRESS)
    from_address = shippo.Address.create(api_key=api_key, **FROM_ADDRESS)
    parcel = shippo.Parcel.create(api_key=api_key, **DUMMY_PARCEL)
    SHIPMENT = DUMMY_SHIPMENT.copy()
    SHIPMENT['address_from'] = from_address.object_id
    SHIPMENT['address_to'] = to_address.object_id
    SHIPMENT['parcels'] = [parcel.object_id]
    SHIPMENT['asynchronous'] = asynchronous
    shipment = shippo.Shipment.create(api_key=api_key, **SHIPMENT)
    return shipment


def create_mock_manifest(transaction=None):
    if not transaction:
        transaction = create_mock_transaction()
    rate = shippo.Rate.retrieve(transaction.rate)
    shipment = shippo.Shipment.retrieve(rate.shipment)
    MANIFEST = DUMMY_MANIFEST.copy()
    MANIFEST['address_from'] = shipment.address_from
    MANIFEST['async'] = False
    manifest = shippo.Manifest.create(**MANIFEST)
    return manifest


def create_mock_transaction(asynchronous=False):
    shipment = create_mock_shipment(asynchronous)
    rates = shipment.rates
    usps_rate = list(
        [x for x in rates if x.servicelevel.token == 'usps_priority'])[0]
    t = DUMMY_TRANSACTION.copy()
    t['rate'] = usps_rate.object_id
    t['asynchronous'] = asynchronous
    txn = shippo.Transaction.create(**t)
    return txn


def create_mock_international_shipment():
    SHIPMENT = create_mock_shipment()
    customs_item = shippo.CustomsItem.create(**DUMMY_CUSTOMS_ITEM)
    customs_declaration_parameters = DUMMY_CUSTOMS_DECLARATION.copy()
    customs_declaration_parameters["items"][0] = customs_item.object_id
    customs_declaration = shippo.CustomsDeclaration.create(
        **customs_declaration_parameters)
    SHIPMENT['customs_declaration'] = customs_declaration.object_id
    shipment = shippo.Shipment.create(**SHIPMENT)
    return shipment

def create_mock_international_transaction(asynchronous=False):
    shipment = create_mock_shipment(asynchronous)
    rates = shipment.rates
    usps_rate = list(
        [x for x in rates if x.servicelevel.token == 'usps_priority'])[0]
    t = DUMMY_TRANSACTION.copy()
    t['rate'] = usps_rate.object_id
    t['asynchronous'] = asynchronous
    txn = shippo.Transaction.create(**t)
    return txn, usps_rate.carrier_account

class ShippoTestCase(TestCase):
    RESTORE_ATTRIBUTES = ('api_version', 'api_key')

    def setUp(self):
        super(ShippoTestCase, self).setUp()

        self._shippo_original_attributes = {}

        for attr in self.RESTORE_ATTRIBUTES:
            self._shippo_original_attributes[attr] = getattr(config, attr)
        api_base = os.environ.get('SHIPPO_API_BASE')
        if api_base:
            config.api_base = api_base

        config.api_key = os.environ.get('SHIPPO_API_KEY', '51895b669caa45038110fd4074e61e0d')
        config.api_version = os.environ.get('SHIPPO_API_VERSION', '2018-02-08')
        config.app_name = os.environ.get('APP_NAME', 'MyAwesomeApp')
        config.app_version = os.environ.get('APP_VERSION', '1.0.0')

    def tearDown(self):
        super(ShippoTestCase, self).tearDown()

        for attr in self.RESTORE_ATTRIBUTES:
            setattr(config, attr, self._shippo_original_attributes[attr])

    # Python < 2.7 compatibility
    def assertRaisesRegexp(self, exception, regexp, callable, *args, **kwargs):
        try:
            callable(*args, **kwargs)
        except exception as err:
            if regexp is None:
                return True

            if isinstance(regexp, str):
                regexp = re.compile(regexp)
            if not regexp.search(str(err)):
                raise self.failureException('"%s" does not match "%s"' %
                                            (regexp.pattern, str(err)))
        else:
            raise self.failureException(
                '%s was not raised' % (exception.__name__,))


class ShippoUnitTestCase(ShippoTestCase):
    REQUEST_LIBRARIES = ['urlfetch', 'requests']

    def setUp(self):
        super(ShippoUnitTestCase, self).setUp()

        self.request_patchers = {}
        self.request_mocks = {}
        for lib in self.REQUEST_LIBRARIES:
            patcher = patch("shippo.http_client.%s" % (lib,))

            self.request_mocks[lib] = patcher.start()
            self.request_patchers[lib] = patcher

    def tearDown(self):
        super(ShippoUnitTestCase, self).tearDown()

        for patcher in list(self.request_patchers.values()):
            patcher.stop()


class ShippoApiTestCase(ShippoTestCase):

    def setUp(self):
        super(ShippoApiTestCase, self).setUp()

        self.requestor_patcher = patch('shippo.api_requestor.APIRequestor')
        requestor_class_mock = self.requestor_patcher.start()
        self.requestor_mock = requestor_class_mock.return_value

    def tearDown(self):
        super(ShippoApiTestCase, self).tearDown()

        self.requestor_patcher.stop()

    def mock_response(self, res):
        self.requestor_mock.request = Mock(return_value=(res, 'reskey'))


shippo_vcr = vcr.VCR(
    filter_headers=['Authorization'],
    record_mode=config.vcr_record_mode
)



import logging

from shippo.config import config

logger = logging.getLogger('shippo')

logging.basicConfig()
vcr_log = logging.getLogger("vcr")
vcr_log.setLevel(config.vcr_logging_level)


__all__ = ['json']

try:
    import json
except ImportError:
    json = None

if not (json and hasattr(json, 'loads')):
    try:
        import simplejson as json
    except ImportError:
        if not json:
            raise ImportError(
                "Shippo requires a JSON library, such as simplejson. "
                "HINT: Try installing the "
                "python simplejson library via 'pip install simplejson' or "
                "contact support@goshippo.com with questions.")
        else:
            raise ImportError(
                "Shippo requires a JSON library with the same interface as "
                "the Python 2.6 'json' library.  You appear to have a 'json' "
                "library with a different interface.  Please install "
                "the simplejson library.  HINT: Try installing the "
                "python simplejson library via 'pip install simplejson' "
                "contact support@goshippo.com with questions.")




from shippo import api_requestor, config
from mock import patch, Mock
from unittest import TestCase
import unittest
import sys

from shippo.api_requestor import APIRequestor
from shippo.config import config, Configuration
from shippo.error import ConfigurationError


class APIRequestorTests(TestCase):
    @patch.object(sys, 'version', '3.8.1 (default, Mar 13 2020, 20:31:03) \n[Clang 11.0.0 (clang-1100.0.33.17)]')
    def test_shippo_user_agent(self):
        configuration = Configuration()
        configuration.app_name = 'TestApp'
        configuration.app_version = '1.1.1'
        configuration.sdk_version = '0.0.0'

        actual = APIRequestor.get_shippo_user_agent_header(configuration=configuration)
        expected = 'TestApp/1.1.1 ShippoPythonSDK/0.0.0 Python/3.8.1'
        self.assertEqual(actual, expected)

    def test_oauth_token_auth(self):
        config.app_name = 'TestApp'
        config.app_version = '1.1.1'

        mock_client = Mock()
        mock_client.name = 'mock_client'
        mock_client.request.return_value = ('{"status": "ok"}', 200)

        requestor = api_requestor.APIRequestor(
            key='oauth.mocktoken.mocksig', client=mock_client)
        requestor.request('GET', '/v1/echo')

        args, kwargs = mock_client.request.call_args
        method, url, headers, data = args

        self.assertDictContainsSubset(
            {'Authorization': 'Bearer oauth.mocktoken.mocksig'},
            headers,
            "Expect correct token type to used for authorization with oauth token"
        )

    def test_shippo_token_auth(self):
        config.app_name = 'TestApp'
        config.app_version = '1.1.1'

        mock_client = Mock()
        mock_client.name = 'mock_client'
        mock_client.request.return_value = ('{"status": "ok"}', 200)

        requestor = api_requestor.APIRequestor(
            key='shippo_test_mocktoken', client=mock_client)
        requestor.request('GET', '/v1/echo')

        args, kwargs = mock_client.request.call_args
        method, url, headers, data = args

        self.assertDictContainsSubset(
            {'Authorization': 'ShippoToken shippo_test_mocktoken'},
            headers,
            "Expect correct token type to used for authorization with shippo token"
        )


if __name__ == '__main__':
    unittest.main()



import calendar
import datetime
import logging
import os
import socket
import ssl
import time
import urllib.parse
import warnings
import sys
from shippo import error, http_client, util, certificate_blacklist
from shippo.config import config, Configuration
from shippo.error import ConfigurationError
from shippo.version import VERSION


def _encode_datetime(dttime):
    if dttime.tzinfo and dttime.tzinfo.utcoffset(dttime) is not None:
        utc_timestamp = calendar.timegm(dttime.utctimetuple())
    else:
        utc_timestamp = time.mktime(dttime.timetuple())

    return int(utc_timestamp)


def _api_encode(data):
    for key, value in list(data.items()):
        key = key
        if value is None:
            continue
        elif hasattr(value, 'shippo_id'):
            yield (key, value.shippo_id)
        elif isinstance(value, list) or isinstance(value, tuple):
            for subvalue in value:
                yield ("%s[]" % (key,), subvalue)
        elif isinstance(value, dict):
            subdict = dict(('%s[%s]' % (key, subkey), subvalue) for
                           subkey, subvalue in list(value.items()))
            for subkey, subvalue in _api_encode(subdict):
                yield (subkey, subvalue)
        elif isinstance(value, datetime.datetime):
            yield (key, _encode_datetime(value))
        else:
            yield (key, value)


def _build_api_url(url, query):
    scheme, netloc, path, base_query, fragment = urllib.parse.urlsplit(url)

    if base_query:
        query = '%s&%s' % (base_query, query)

    return urllib.parse.urlunsplit((scheme, netloc, path, query, fragment))


class APIRequestor(object):

    _CERTIFICATE_VERIFIED = False

    def __init__(self, key=None, client=None):
        self.api_key = key

        self._client = client or http_client.new_default_http_client(
            verify_ssl_certs=config.verify_ssl_certs,
            timeout_in_seconds=config.timeout_in_seconds)

    def request(self, method, url, params=None):
        self._check_ssl_cert()\

        if params is not None and isinstance(params, dict):
            params = {('async' if k == 'asynchronous' else k): v for k, v in params.items()}

        rbody, rcode, my_api_key = self.request_raw(
            method.lower(), url, params)

        resp = self.interpret_response(rbody, rcode)
        return resp, my_api_key

    def handle_api_error(self, rbody, rcode, resp):

        if rcode in [400, 404]:
            raise error.InvalidRequestError(rbody, rcode, resp)
        elif rcode == 401:
            raise error.AuthenticationError(rbody, rcode, resp)
        else:
            raise error.APIError(rbody, rcode, resp)

    @staticmethod
    def get_python_version() -> str:
        return sys.version.split(' ')[0]

    @staticmethod
    def get_shippo_user_agent_header(configuration: Configuration) -> str:
        key_value_pairs = []
        key_value_pairs.append('/'.join([configuration.app_name, configuration.app_version]))
        key_value_pairs.append('/'.join([configuration.sdk_name, configuration.sdk_version]))
        key_value_pairs.append('/'.join([configuration.language, APIRequestor.get_python_version()]))
        return ' '.join(key_value_pairs)

    def request_raw(self, method, url, params=None):
        """
        Mechanism for issuing an API call
        """

        if self.api_key:
            my_api_key = self.api_key
        else:
            my_api_key = config.api_key

        if my_api_key is None:
            raise error.AuthenticationError(
                'No API key provided. (HINT: set your API key using '
                '"shippo.config.api_key = shippo_test_d90f00698a0a8def0495fddb4212bb08051469d3"). You can generate API keys '
                'from the Shippo web interface.  See https://goshippo.com/api '
                'for details, or email support@goshippo.com if you have any '
                'questions.')

        token_type = 'ShippoToken'
        if my_api_key.startswith('oauth.'):
            token_type = 'Bearer'

        abs_url = '%s%s' % (config.api_base, url)

        if method == 'get' or method == 'delete':
            if params:
                encoded_params = urllib.parse.urlencode(
                    list(_api_encode(params or {})))
                abs_url = _build_api_url(abs_url, encoded_params)
            post_data = None
        elif method == 'post' or method == 'put':
            post_data = util.json.dumps(params)
        else:
            raise error.APIConnectionError(
                'Unrecognized HTTP method %r.  This may indicate a bug in the '
                'Shippo bindings.  Please contact support@goshippo.com for '
                'assistance.' % (method,))

        shippo_user_agent = APIRequestor.get_shippo_user_agent_header(config)

        headers = {
            'Content-Type': 'application/json',
            'X-Shippo-Client-User-Agent': shippo_user_agent,
            'User-Agent': '%s/%s ShippoPythonSDK/%s' % (config.app_name, config.app_version, config.sdk_version, ),
            'Authorization': '%s %s' % (token_type, my_api_key,),
            'Shippo-API-Version': config.api_version
        }

        rbody, rcode = self._client.request(
            method, abs_url, headers, post_data)
        util.logger.info(
            'API request to %s returned (response code, response body) of '
            '(%d, %r)',
            abs_url, rcode, rbody)
        return rbody, rcode, my_api_key

    def interpret_response(self, rbody, rcode):
        try:
            if hasattr(rbody, 'decode'):
                rbody = rbody.decode('utf-8')
                if rbody == '':
                    rbody = '{"msg": "empty_response"}'
            resp = util.json.loads(rbody)                
        except Exception:
            raise error.APIError(
                "Invalid response body from API: %s "
                "(HTTP response code was %d)" % (rbody, rcode),
                rbody, rcode)
        if not (200 <= rcode < 300):
            self.handle_api_error(rbody, rcode, resp)
        return resp

    def _check_ssl_cert(self):
        """Preflight the SSL certificate presented by the backend.

        This isn't 100% bulletproof, in that we're not actually validating the
        transport used to communicate with Shippo, merely that the first
        attempt to does not use a revoked certificate.

        Unfortunately the interface to OpenSSL doesn't make it easy to check
        the certificate before sending potentially sensitive data on the wire.
        This approach raises the bar for an attacker significantly."""

        if config.verify_ssl_certs and not self._CERTIFICATE_VERIFIED:
            uri = urllib.parse.urlparse(config.api_base)
            try:
                certificate = ssl.get_server_certificate(
                    (uri.hostname, uri.port or 443))
                der_cert = ssl.PEM_cert_to_DER_cert(certificate)
            except socket.error as e:
                raise error.APIConnectionError(e)
            except TypeError:
                # The Google App Engine development server blocks the C socket
                # module which causes a type error when using the SSL library
                if ('APPENGINE_RUNTIME' in os.environ and
                        'Dev' in os.environ.get('SERVER_SOFTWARE', '')):
                    self._CERTIFICATE_VERIFIED = True
                    warnings.warn(
                        'We were unable to verify Shippo\'s SSL certificate '
                        'due to a bug in the Google App Engine development '
                        'server. Please alert us immediately at '
                        'suppgoshippo.compo.com if this message appears in your '
                        'production logs.')
                    return
                else:
                    raise

            self._CERTIFICATE_VERIFIED = certificate_blacklist.verify(
                uri.hostname, der_cert)



import os
from setuptools import setup


version_contents = {}

here = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(here, "shippo", "version.py"), encoding="utf-8") as f:
    exec(f.read(), version_contents)

setup(
    name='shippo',
    version=version_contents['VERSION'],
    description='Shipping API Python library (USPS, FedEx, UPS and more)',
    author='Shippo',
    author_email='support@goshippo.com',
    url='https://goshippo.com/',
    packages=['shippo', 'shippo.test', 'shippo.test.integration'],
    package_data={'shippo': ['../VERSION']},
    install_requires=[
        'chardet',
        'requests',
        'simplejson >= 3.16.0, <= 3.17.2',
    ],
    test_suite='shippo.test.all',
    tests_require=['mock', 'vcrpy'],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ]
)
